# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
from decimal import Decimal
from unittest import TestResult

from django.core import exceptions
from django.core.files.base import ContentFile
from django.db.models import fields as django_fields
from django.test import TestCase as DjangoTestCase

from testtools import (
    ExtendedToOriginalDecorator,
    TestCase,
    )
from testtools.matchers import StartsWith

from django_factory import generators, testcase
from django_factory.factory import DummyTestCase, Factory
from django_project.factory_tests import models as test_models


class TestGenerators(TestCase):

    def setUp(self):
        super(TestGenerators, self).setUp()
        self.field = test_models.ModelWithCharField._meta.get_field(
                'char_field')
        self.factory = Factory()

    def test_generate_boolean(self):
        generated = generators.generate_boolean(self.field, self.factory)
        self.assertIs(False, generated)

    def test_generate_unicode_generates_unicode(self):
        generated = generators.generate_unicode(self.field, self.factory)
        self.assertIsInstance(generated, unicode)

    def test_generate_unicode_starts_with_field_name(self):
        generated = generators.generate_unicode(self.field, self.factory)
        self.assertThat(generated, StartsWith(self.field.name))

    def test_generate_unicode_respects_max_length(self):
        field = test_models.ModelWithMaxLengthCharField._meta.get_field(
                'char_field')
        generated = generators.generate_unicode(field, self.factory)
        self.assertEquals(field.max_length, len(generated))

    def test_generate_unicode_handles_None_max_length(self):
        field = test_models.ModelWithNoneMaxLengthTextField._meta.get_field(
                'text_field')
        generated = generators.generate_unicode(field, self.factory)
        self.assertIsInstance(generated, unicode)

    def test_generate_int_generates_int(self):
        generated = generators.generate_int(self.field, self.factory)
        self.assertIsInstance(generated, int)

    def test_generate_float_generates_float(self):
        generated = generators.generate_float(self.field, self.factory)
        self.assertIsInstance(generated, float)

    def test_generate_decimal_generates_decimal(self):
        generated = generators.generate_decimal(self.field, self.factory)
        self.assertIsInstance(generated, Decimal)

    def test_generate_date_generates_date(self):
        generated = generators.generate_date(self.field, self.factory)
        self.assertIsInstance(generated, datetime.date)

    def test_generate_datetime_generates_datetime(self):
        generated = generators.generate_datetime(self.field, self.factory)
        self.assertIsInstance(generated, datetime.datetime)

    def test_generate_time_generates_time(self):
        generated = generators.generate_time(self.field, self.factory)
        self.assertIsInstance(generated, datetime.time)

    def test_generate_url_generates_unicode(self):
        generated = generators.generate_url(self.field, self.factory)
        self.assertIsInstance(generated, unicode)

    def test_generate_url_starts_with_http(self):
        generated = generators.generate_url(self.field, self.factory)
        self.assertThat(generated, StartsWith("http://"))

    def test_generate_email_generates_unicode(self):
        generated = generators.generate_email(self.field, self.factory)
        self.assertIsInstance(generated, unicode)

    def test_generate_email_contains_at(self):
        generated = generators.generate_email(self.field, self.factory)
        self.assertIn('@', generated)

    def test_generate_comma_separated_integer(self):
        generated = generators.generate_comma_separated_integer(self.field,
                self.factory)
        self.assertIsInstance(generated, str)

    def test_generate_comma_separated_integer_generates_str_int(self):
        generated = generators.generate_comma_separated_integer(self.field,
                self.factory)
        # Check that we can call int on the result
        int(generated)

    def test_generate_ip_address(self):
        generated = generators.generate_ip_address(self.field, self.factory)
        self.assertIsInstance(generated, str)

    def test_generate_ip_address_value(self):
        generated = generators.generate_ip_address(self.field, self.factory)
        parts = generated.split(".")
        self.assertEqual(4, len(parts))
        map(int, parts)

    def test_generate_generic_ip_address(self):
        generated = generators.generate_generic_ip_address(self.field, self.factory)
        self.assertIsInstance(generated, str)

    def test_generate_generic_ip_address_value(self):
        generated = generators.generate_generic_ip_address(self.field, self.factory)
        parts = generated.split("::")
        self.assertEqual(2, len(parts))

    def test_generate_file_generates_content_file(self):
        generated = generators.generate_file(self.field, self.factory)
        self.assertIsInstance(generated, ContentFile)

    def test_generate_file_contents_start_with_field_name(self):
        generated = generators.generate_file(self.field, self.factory)
        self.assertThat(generated.read(), StartsWith(self.field.name))


class FactoryGenerateValueTests(DjangoTestCase, TestCase):

    def get_factory_with_known_generator(self, field, expected, value):
        """Get a factory replacing the generator for the particular field class.

        Checks first that the default generator is as expected, and then
        creates a Factory that replaces the default with a function
        that returns the given value.

        :param field: the Field class that is being tested
        :param expected: the expected default generator for that class
        :param value: the value that the factory should return when
                generating a value for the Field class.
        :return: a Factory instance that will return ``value`` when
                generating a value for an instance of ``field``.
        :raises AssertionError: if the default generator for ``field``
                is not equal to ``expected``.
        """
        self.assertEqual(expected, Factory.DEFAULT_FIELD_MAPPING[field])
        mapping = Factory.DEFAULT_FIELD_MAPPING.copy()
        def return_value(_field, _factory):
            return value
        mapping[field] = return_value
        return Factory(field_mapping=mapping)

    def get_field(self, model, name):
        """Get the Field instance of the particular name from the model.

        Given a model this method will return the Field instance that
        is assigned the the named attribute on the model.

        :param model: the Model instance to take the field from.
        :param name: the attribute on the model that the field should
                be taken from.
        :return: The ``Field`` subclass that is assigned to ``name`` on
                ``model``.
        """
        return model._meta.get_field(name, many_to_many=True)

    def assertGeneratesValidValueFor(self, field):
        """Check that the value created for ``field`` passes validation.

        This asks the factory to generate a value for ``field`` using the default
        generator, and then checks the the value passes ``fields`` validators.

        :param field: the Field instance to check the value generated for.
        :return: None
        :raises ValidationError: If the value produced for ``field`` doesn't
            pass validation.
        :raise SkipTest: if the version of django in use doesn't support
            validators.
        """
        if getattr(django_fields.CharField, 'default_validators', None) is None:
            self.skip("This version of django doesn't support validators")
        factory = Factory()
        value = factory.generate_value(field)
        field.run_validators(value)

    def test_boolean_field(self):
        expected_value = True
        factory = self.get_factory_with_known_generator(
                django_fields.BooleanField, generators.generate_boolean,
                expected_value)
        model = test_models.ModelWithBooleanField
        field = self.get_field(model, 'bool_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_boolean_field_generates_valid_value(self):
        model = test_models.ModelWithBooleanField
        field = self.get_field(model, 'bool_field')
        self.assertGeneratesValidValueFor(field)

    def test_null_boolean_field(self):
        expected_value = True
        factory = self.get_factory_with_known_generator(
                django_fields.NullBooleanField, generators.generate_boolean,
                expected_value)
        model = test_models.ModelWithNullBooleanField
        field = self.get_field(model, 'bool_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_null_boolean_field_generates_valid_value(self):
        model = test_models.ModelWithNullBooleanField
        field = self.get_field(model, 'bool_field')
        self.assertGeneratesValidValueFor(field)

    def test_int_field(self):
        expected_value = 11111
        factory = self.get_factory_with_known_generator(
                django_fields.IntegerField, generators.generate_int,
                expected_value)
        model = test_models.ModelWithIntegerField
        field = self.get_field(model, 'int_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_int_field_generates_valid_value(self):
        model = test_models.ModelWithIntegerField
        field = self.get_field(model, 'int_field')
        self.assertGeneratesValidValueFor(field)

    def test_big_int_field(self):
        if getattr(django_fields, 'BigIntegerField', None) is None:
            self.skip("This version of django doesn't have BigIntegerField")
        expected_value = 11111
        factory = self.get_factory_with_known_generator(
                django_fields.BigIntegerField, generators.generate_int,
                expected_value)
        model = test_models.ModelWithBigIntegerField
        field = self.get_field(model, 'big_int_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_big_int_field_generates_valid_value(self):
        if getattr(django_fields, 'BigIntegerField', None) is None:
            self.skip("This version of django doesn't have BigIntegerField")
        model = test_models.ModelWithBigIntegerField
        field = self.get_field(model, 'big_int_field')
        self.assertGeneratesValidValueFor(field)

    def test_small_int_field(self):
        expected_value = 11111
        factory = self.get_factory_with_known_generator(
                django_fields.SmallIntegerField, generators.generate_int,
                expected_value)
        model = test_models.ModelWithSmallIntegerField
        field = self.get_field(model, 'small_int_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_small_int_field_generates_valid_value(self):
        model = test_models.ModelWithSmallIntegerField
        field = self.get_field(model, 'small_int_field')
        self.assertGeneratesValidValueFor(field)

    def test_positive_int_field(self):
        expected_value = 11111
        factory = self.get_factory_with_known_generator(
                django_fields.PositiveIntegerField, generators.generate_int,
                expected_value)
        model = test_models.ModelWithPositiveIntegerField
        field = self.get_field(model, 'positive_int_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_positive_int_field_generates_valid_value(self):
        model = test_models.ModelWithPositiveIntegerField
        field = self.get_field(model, 'positive_int_field')
        self.assertGeneratesValidValueFor(field)

    def test_positive_small_int_field(self):
        expected_value = 11111
        factory = self.get_factory_with_known_generator(
                django_fields.PositiveSmallIntegerField,
                generators.generate_int, expected_value)
        model = test_models.ModelWithPositiveSmallIntegerField
        field = self.get_field(model, 'positive_small_int_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_positive_small_int_field_generates_valid_value(self):
        model = test_models.ModelWithPositiveSmallIntegerField
        field = self.get_field(model, 'positive_small_int_field')
        self.assertGeneratesValidValueFor(field)

    def test_float_field(self):
        expected_value = 111.11
        factory = self.get_factory_with_known_generator(
                django_fields.FloatField,
                generators.generate_float, expected_value)
        model = test_models.ModelWithFloatField
        field = self.get_field(model, 'float_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_float_field_generates_valid_value(self):
        model = test_models.ModelWithFloatField
        field = self.get_field(model, 'float_field')
        self.assertGeneratesValidValueFor(field)

    def test_decimal_field(self):
        expected_value = Decimal(1111)
        factory = self.get_factory_with_known_generator(
                django_fields.DecimalField,
                generators.generate_decimal, expected_value)
        model = test_models.ModelWithDecimalField
        field = self.get_field(model, 'decimal_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_decimal_field_generates_valid_value(self):
        model = test_models.ModelWithDecimalField
        field = self.get_field(model, 'decimal_field')
        self.assertGeneratesValidValueFor(field)

    def test_char_field(self):
        expected_value = "aaaaaa"
        factory = self.get_factory_with_known_generator(
                django_fields.CharField,
                generators.generate_unicode, expected_value)
        model = test_models.ModelWithCharField
        field = self.get_field(model, 'char_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_char_field_generates_valid_value(self):
        model = test_models.ModelWithCharField
        field = self.get_field(model, 'char_field')
        self.assertGeneratesValidValueFor(field)

    def test_text_field(self):
        expected_value = "aaaaaa"
        factory = self.get_factory_with_known_generator(
                django_fields.TextField,
                generators.generate_unicode, expected_value)
        model = test_models.ModelWithTextField
        field = self.get_field(model, 'text_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_text_field_generates_valid_value(self):
        model = test_models.ModelWithTextField
        field = self.get_field(model, 'text_field')
        self.assertGeneratesValidValueFor(field)

    def test_slug_field(self):
        expected_value = "aaaaaa"
        factory = self.get_factory_with_known_generator(
                django_fields.SlugField,
                generators.generate_unicode, expected_value)
        model = test_models.ModelWithSlugField
        field = self.get_field(model, 'slug_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_slug_field_generates_valid_value(self):
        model = test_models.ModelWithSlugField
        field = self.get_field(model, 'slug_field')
        self.assertGeneratesValidValueFor(field)

    def test_date_field(self):
        expected_value = datetime.date(2010, 10, 10)
        factory = self.get_factory_with_known_generator(
                django_fields.DateField,
                generators.generate_date, expected_value)
        model = test_models.ModelWithDateField
        field = self.get_field(model, 'date_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_date_field_generates_valid_value(self):
        model = test_models.ModelWithDateField
        field = self.get_field(model, 'date_field')
        self.assertGeneratesValidValueFor(field)

    def test_datetime_field(self):
        expected_value = datetime.datetime(2010, 10, 10)
        factory = self.get_factory_with_known_generator(
                django_fields.DateTimeField,
                generators.generate_datetime, expected_value)
        model = test_models.ModelWithDateTimeField
        field = self.get_field(model, 'datetime_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_datetime_field_generates_valid_value(self):
        model = test_models.ModelWithDateTimeField
        field = self.get_field(model, 'datetime_field')
        self.assertGeneratesValidValueFor(field)

    def test_time_field(self):
        expected_value = datetime.time(10, 10)
        factory = self.get_factory_with_known_generator(
                django_fields.TimeField,
                generators.generate_time, expected_value)
        model = test_models.ModelWithTimeField
        field = self.get_field(model, 'time_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_time_field_generates_valid_value(self):
        model = test_models.ModelWithTimeField
        field = self.get_field(model, 'time_field')
        self.assertGeneratesValidValueFor(field)

    def test_url_field_default_generator(self):
        expected_value = "http://example.com/aaaaaa"
        factory = self.get_factory_with_known_generator(
                django_fields.URLField,
                generators.generate_url, expected_value)
        model = test_models.ModelWithURLField
        field = self.get_field(model, 'url_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_url_field_generates_valid_value(self):
        model = test_models.ModelWithURLField
        field = self.get_field(model, 'url_field')
        self.assertGeneratesValidValueFor(field)

    def test_email_field(self):
        expected_value = "aaaaa@example.com"
        factory = self.get_factory_with_known_generator(
                django_fields.EmailField,
                generators.generate_email, expected_value)
        model = test_models.ModelWithEmailField
        field = self.get_field(model, 'email_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_email_field_generates_valid_value(self):
        model = test_models.ModelWithEmailField
        field = self.get_field(model, 'email_field')
        self.assertGeneratesValidValueFor(field)

    def test_comma_separated_integer_field(self):
        expected_value = "1,2,3"
        factory = self.get_factory_with_known_generator(
                django_fields.CommaSeparatedIntegerField,
                generators.generate_comma_separated_integer,
                expected_value)
        model = test_models.ModelWithCommaSeparatedIntegerField
        field = self.get_field(model, 'csi_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_comma_separated_integer_field_generates_valid_value(self):
        model = test_models.ModelWithCommaSeparatedIntegerField
        field = self.get_field(model, 'csi_field')
        self.assertGeneratesValidValueFor(field)

    def test_ip_address_field(self):
        expected_value = "192.111.10.1"
        factory = self.get_factory_with_known_generator(
                django_fields.IPAddressField,
                generators.generate_ip_address,
                expected_value)
        model = test_models.ModelWithIPAddressField
        field = self.get_field(model, 'ip_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_ip_address_field_generates_valid_value(self):
        model = test_models.ModelWithIPAddressField
        field = self.get_field(model, 'ip_field')
        self.assertGeneratesValidValueFor(field)

    def test_generic_ip_address_field(self):
        if getattr(django_fields, 'GenericIPAddressField', None) is None:
            self.skip("This version of django has no GenericIPAddressField")
        expected_value = "192.111.10.1"
        factory = self.get_factory_with_known_generator(
                django_fields.GenericIPAddressField,
                generators.generate_generic_ip_address,
                expected_value)
        model = test_models.ModelWithGenericIPAddressField
        field = self.get_field(model, 'ip_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_generic_ip_address_field_generates_valid_value(self):
        if getattr(django_fields, 'GenericIPAddressField', None) is None:
            self.skip("This version of django has no GenericIPAddressField")
        model = test_models.ModelWithGenericIPAddressField
        field = self.get_field(model, 'ip_field')
        self.assertGeneratesValidValueFor(field)

    def test_file_field(self):
        expected_value = ContentFile("aaa")
        factory = self.get_factory_with_known_generator(
                django_fields.files.FileField,
                generators.generate_file,
                expected_value)
        model = test_models.ModelWithFileField
        field = self.get_field(model, 'file_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_file_field_generates_valid_value(self):
        model = test_models.ModelWithFileField
        field = self.get_field(model, 'file_field')
        self.assertGeneratesValidValueFor(field)

    def test_image_field(self):
        expected_value = ContentFile("aa")
        factory = self.get_factory_with_known_generator(
                django_fields.files.ImageField,
                generators.generate_file,
                expected_value)
        model = test_models.ModelWithImageField
        field = self.get_field(model, 'image_field')
        value = factory.generate_value(field)
        self.assertEqual(expected_value, value)

    def test_image_field_generates_valid_value(self):
        model = test_models.ModelWithImageField
        field = self.get_field(model, 'image_field')
        self.assertGeneratesValidValueFor(field)

    def test_chooses_from_choices(self):
        # When a field has choices, those choices are used when
        # generating a value for that field.
        factory = Factory()
        model = test_models.ModelWithChoices
        field = self.get_field(model, 'choices_field')
        value = factory.generate_value(field)
        self.assertEqual(test_models.ModelWithChoices.CHOICES[0][0],
                value)

    def test_chooses_from_choices_with_groups(self):
        # Still selects the first choice, even if there are choice
        # groups.
        factory = Factory()
        model = test_models.ModelWithChoiceGroups
        field = self.get_field(model, 'choices_field')
        value = factory.generate_value(field)
        self.assertEqual(test_models.ModelWithChoiceGroups.CHOICES[0][1][0][0],
                value)

    def test_type_error_on_unknown_field(self):
        factory = Factory()
        class Field(django_fields.Field):
            pass
        field = Field()
        self.assertRaises(TypeError, factory.generate_value, field)


class TestFactory(TestCase, DjangoTestCase):

    def test_get_fields_with_no_fields(self):
        fields = Factory.get_fields(test_models.ModelWithNoFields)
        self.assertEqual([], fields)

    def test_get_fields_returns_normal_fields(self):
        fields = Factory.get_fields(test_models.ModelWithCharField)
        self.assertEqual([test_models.ModelWithCharField._meta.get_field_by_name('char_field')[0]], fields)

    def test_get_fields_returns_many_to_many_field(self):
        fields = Factory.get_fields(test_models.ModelWithSelfManyToManyField)
        self.assertEqual([test_models.ModelWithSelfManyToManyField._meta.get_field_by_name('things')[0]], fields)

    def test_make_one_makes_one(self):
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNoFields)
        self.assertIsInstance(instance, test_models.ModelWithNoFields)

    def test_make_one_saves(self):
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNoFields)
        # Raises DoesNotExist if it wasn't saved
        test_models.ModelWithNoFields.objects.filter(pk=instance.pk).get()

    def test_prepare_one_makes_one(self):
        factory = Factory()
        instance = factory.prepare_one(test_models.ModelWithNoFields)
        self.assertIsInstance(instance, test_models.ModelWithNoFields)

    def test_prepare_one_doesnt_save(self):
        factory = Factory()
        instance = factory.prepare_one(test_models.ModelWithNoFields)
        self.assertRaises(test_models.ModelWithNoFields.DoesNotExist,
            test_models.ModelWithNoFields.objects.filter(pk=instance.pk).get)

    def test_make_one_generates_value(self):
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithCharField)
        self.assertIsInstance(instance.char_field, unicode)
        self.assert_("NOT_PROVIDED" not in instance.char_field)

    def test_make_one_uses_passed_value(self):
        factory = Factory()
        value = u'bar'
        instance = factory.make_one(test_models.ModelWithCharField, char_field=value)
        self.assertEquals(value, instance.char_field)

    def test_make_one_doesnt_generate_for_null_and_blank(self):
        # When a field has null=True and blank=True, no value is generated
        # for it by default.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNullAndBlankCharField)
        self.assertEquals(None, instance.char_field)

    def test_make_one_doesnt_generate_for_null_only(self):
        # When a field has null=True and not blank=True, a value is generated
        # for it by default.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNullCharField)
        self.assertIsInstance(instance.char_field, unicode)

    def test_make_one_doesnt_generate_for_blank_only(self):
        # When a field has blank=True and not null=True, a value is generated
        # for it by default.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithBlankCharField)
        self.assertIsInstance(instance.char_field, unicode)

    def test_make_one_doesnt_generates_for_null_if_factory_method(self):
        # If there is a factory method on the model for a field, that is
        # used even if the field has null=True and blank=True
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNullAndBlankCharFieldAndFactory)
        self.assertEquals(test_models.ModelWithNullAndBlankCharFieldAndFactory.VALUE,
                instance.char_field)

    def test_make_one_uses_default(self):
        # When a field has a default set, that is used when generating a
        # value for that field.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithDefaultCharField)
        self.assertEquals(test_models.ModelWithDefaultCharField.DEFAULT,
                instance.char_field)

    def test_make_one_uses_callable_default(self):
        # If a field has a callable default, that is correctly handled
        # by using the return value of the callable.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithCallableDefaultCharField)
        self.assertEquals(test_models.ModelWithCallableDefaultCharField.DEFAULT,
                instance.char_field)

    def test_make_one_doesnt_use_default_if_factory_method(self):
        # If there is a factory generator method for a field, that is
        # used in preference to any default.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithDefaultCharFieldAndFactory)
        self.assertEquals(test_models.ModelWithDefaultCharFieldAndFactory.VALUE,
                instance.char_field)

    def test_make_one_uses_field_factory(self):
        # Any factory generator method for a field is used to get the
        # value for that field.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithFieldGenerator)
        self.assertEquals(test_models.ModelWithFieldGenerator.VALUE,
                instance.char_field)

    def test_make_one_uses_passed_value_over_factory(self):
        # Any value passed to make_one will be used in preference to any
        # generator for that field.
        factory = Factory()
        value = 'aaaa'
        instance = factory.make_one(test_models.ModelWithFieldGenerator,
                char_field=value)
        self.assertEquals(value, instance.char_field)

    def test_make_one_uses_generator_for_instance(self):
        # If a model has a factory instance generator that is used when
        # make_one is called for that model.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithInstanceGenerator)
        self.assertEqual(test_models.ModelWithFieldGenerator.VALUE,
                instance.char_field)

    def test_make_one_validates_model(self):
        # Any validators on the model are checked before the model is
        # returned in order to save surprises.
        # validators are new in django 1.2, so skip the test on earlier
        # releases
        if getattr(django_fields.CharField, 'default_validators', None) is None:
            self.skip("This version of django doesn't support validators")
        factory = Factory()
        self.assertRaises(exceptions.ValidationError,
                factory.make_one, test_models.ModelWithImpossibleValidator)

    def test_prepare_one_doesnt_validate_model(self):
        # When calling prepare_one the validators aren't run, as
        # foreignkey constraints can't be properly satisfied if
        # the model isn't being saved
        if getattr(django_fields.CharField, 'default_validators', None) is None:
            self.skip("This version of django doesn't support validators")
        factory = Factory()
        instance = factory.prepare_one(test_models.ModelWithImpossibleValidator)
        self.assertIsInstance(instance,
                test_models.ModelWithImpossibleValidator)

    def test_make_one_makes_one_of_many_to_many_target(self):
        # If there is a ManyToManyField on the model, one instance of
        # the target is generated.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithManyToManyField)
        self.assertEqual(1, instance.things.count())
        self.assertIsInstance(instance.things.get(),
                test_models.ModelReferencedByManyToManyField)

    def test_make_one_uses_generator_for_many_to_many(self):
        # If the model has a factory generator function for the many to
        # many field then it is used when creating the instance of the
        # ManyToManyField target.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithManyToManyFieldAndFactory)
        self.assertEqual(1, instance.things.count())
        self.assertEqual(test_models.ModelWithManyToManyFieldAndFactory.VALUE,
                instance.things.get())

    def test_make_one_makes_specified_number_of_many_to_many_target(self):
        # If there is a factory attribute specifying the number of
        # instances of a ManyToManyField target to make, then that is
        # obeyed when construcing the instances.
        factory = Factory()
        instance = factory.make_one(
                test_models.ModelWantingTwoManyToManyInstances)
        self.assertEqual(2, instance.things.count())

    def test_make_one_uses_callable_number_of_many_to_many_instances(self):
        # If the factory attribute specifying the number of instances of
        # a ManyToManyField is a callable, then it is called and the
        # resulting number of instances are created.
        factory = Factory()
        instance = factory.make_one(
                test_models.ModelWithCallableSpecifiedManyToManyField)
        self.assertEqual(test_models.ModelWithCallableSpecifiedManyToManyField.COUNT,
                instance.things.count())

    def test_make_one_makes_zero_instances_of_many_to_many_target_if_desired(self):
        # If the model factory specifies zero instances of a
        # ManyToManyField should be created, then don't create any.
        factory = Factory()
        instance = factory.make_one(
                test_models.ModelWantingZeroManyToManyInstances)
        self.assertEqual(0, instance.things.count())

    def test_make_one_uses_value_for_many_to_many_target(self):
        # If a value is passed to make_one for a ManyToManyField then it
        # is used rather than creating any instances.
        factory = Factory()
        target = factory.make_one(test_models.ModelReferencedByManyToManyField)
        instance = factory.make_one(test_models.ModelWithManyToManyFieldAndFactory,
                things=[target])
        self.assertEqual(1, instance.things.count())
        self.assertEqual(target.pk, instance.things.get().pk)

    def test_make_one_with_empty_list_passed(self):
        # If an empty list is passed to make_one for a ManyToManyField then
        # no values are generated
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithManyToManyField,
                things=[])
        self.assertEqual(0, instance.things.count())

    def test_make_one_makes_none_of_many_to_many_target_when_null(self):
        # When null=True is set on a ManyToManyField create no instances
        # of the target model
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNullManyToManyField)
        self.assertEqual(0, instance.things.count())

    def test_make_one_uses_many_to_many_generator_when_null(self):
        # If there is a factory generator function for a ManyToMany
        # field then it is used even when null=True is set on that
        # field.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNullManyToManyFieldAndFactory)
        self.assertEqual(1, instance.things.count())
        self.assertEqual(test_models.ModelWithNullManyToManyFieldAndFactory.VALUE,
                instance.things.get())

    def test_make_one_breaks_many_to_many_reference_loop(self):
        # Any loops between models with ManyToManyField are broken.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithCircularReferenceLoop1)
        self.assertEqual(1, instance.things.count())
        self.assertEqual(0, instance.things.get().other_things.count())

    def test_make_one_breaks_many_to_many_self_reference_loop(self):
        # If a model referenes itself in a ManyToManyField the loop will
        # be broken after creating one target instance for the first
        # instance created.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithSelfManyToManyField)
        self.assertEqual(1, instance.things.count())
        self.assertEqual(0, instance.things.get().things.count())

    def test_make_one_creates_needed_instances_of_through_models(self):
        # When a ManyToManyField has a through model, instances of that
        # through model are created as needed.
        factory = Factory()
        instance = factory.make_one(test_models.Group)
        self.assertEqual(1, instance.people.count())
        person = instance.people.get()
        self.assertIsInstance(person, test_models.Person)
        self.assertEqual(instance, person.group_set.get())
        membership = person.membership_set.get()
        self.assertIsInstance(membership, test_models.Membership)
        self.assertIsInstance(membership.comment, unicode)

    def test_make_one_makes_one_of_foreign_key_target(self):
        # An instance is created for any ForeignKey on the model.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithForeignKey)
        self.assertIsInstance(instance.thing,
                test_models.ModelReferencedByForiegnKeyField)

    def test_make_one_doesnt_make_one_of_foreign_key_if_null_and_blank(self):
        # If null=True and blank=True on a ForeignKey then no instance
        # will be generated.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNullAndBlankForeignKey)
        self.assertEqual(None, instance.thing)

    def test_make_one_makes_one_of_foreign_key_target_if_just_null(self):
        # If just null=True but not blank=True on a ForeignKey then a
        # reference will be generated.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNullForeignKey)
        self.assertIsInstance(instance.thing,
                test_models.ModelReferencedByForiegnKeyField)

    def test_make_one_makes_one_of_foreign_key_target_if_just_blank(self):
        # If just blank=True but not null=True on a ForeignKey then a
        # reference will be generated.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithBlankForeignKey)
        self.assertIsInstance(instance.thing,
                test_models.ModelReferencedByForiegnKeyField)

    def test_make_one_uses_factory_for_foreign_key(self):
        # If there is a factory generator function for a ForeignKey
        # field then it will be used.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithForeignKeyAndFactory)
        self.assertEqual(test_models.ModelWithForeignKeyAndFactory.VALUE, instance.thing)

    def test_make_one_uses_passed_value_for_foreign_key(self):
        # If you pass a value for a ForeignKey then it will be used
        # rather than any factory generator function.
        factory = Factory()
        value = test_models.ModelReferencedByForiegnKeyField()
        instance = factory.make_one(test_models.ModelWithForeignKeyAndFactory,
                thing=value)
        self.assertEqual(value, instance.thing)

    def test_make_one_uses_None_as_passed_value_for_foreign_key(self):
        # If you pass a None value for a ForeignKey then it will be used
        # rather than any factory generator function.
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithNullAndBlankForeignKey,
                thing=None)
        self.assertEqual(None, instance.thing)

    def test_make_one_breaks_foreign_key_reference_loop(self):
        # If there is a loop between models using ForeignKey references,
        # then it will be broken on the model that has null=True and
        # blank=True
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithCircularForiegnKeyReferenceLoop1)
        self.assertIsInstance(instance.thing, test_models.ModelWithCircularForiegnKeyReferenceLoop2)
        self.assertEqual(None, instance.thing.thing)

    def test_make_one_saves_foreign_key_foreign_key(self):
        # When creating an instance needed for foreign keys, it saves
        # the instance, and so can satisfy any foreign key constraints
        # on the foreign key model
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithForiegnKeyToModelWithForeignKey)
        self.assertIsInstance(instance,
                test_models.ModelWithForiegnKeyToModelWithForeignKey)
        self.assertIsInstance(instance.thing, test_models.ModelWithForeignKey)
        self.assertIsInstance(instance.thing.thing,
                test_models.ModelReferencedByForiegnKeyField)

    def test_make_one_makes_fk_for_every_self_needed_for_m2m(self):
        # Where a model with a self m2m field has a foriegn key,
        # instances are created for that foreign key for every
        # self instance created
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithForeignKeyAndSelfManyToMany)
        self.assertIsInstance(instance,
                test_models.ModelWithForeignKeyAndSelfManyToMany)
        self.assertIsInstance(instance.thing,
                test_models.ModelReferencedByForiegnKeyField)
        self.assertEqual(1, instance.selves.count())
        self.assertIsInstance(instance.selves.get(),
                test_models.ModelWithForeignKeyAndSelfManyToMany)
        self.assertIsInstance(instance.selves.get().thing,
                test_models.ModelReferencedByForiegnKeyField)

    def test_make_one_passes_attrs_to_instance_generator(self):
        # Any factory instance generator gets passed the attrs passed to
        # make_one so that it can used them when generating the
        # instance.
        expected_value = 'AAAAA'
        factory = Factory()
        instance = factory.make_one(test_models.ModelWithInstanceGenerator,
                char_field=expected_value)
        self.assertEqual(expected_value, instance.char_field)

    def test_make_one_uses_any_model_generators_entry(self):
        # If there is a model_generators entry for the model being
        # generated then it is used.
        model = test_models.ModelWithCharField
        expected_value = model(char_field=self.getUniqueString())
        def make_value(factory, **attrs):
            return expected_value
        factory = Factory(model_generators={model: make_value})
        instance = factory.make_one(model)
        self.assertIs(expected_value, instance)

    def test_make_makes_many(self):
        factory = Factory()
        count = 5
        instances = factory.make(count, test_models.ModelWithNoFields)
        self.assertEqual(count, len(instances))

    def test_make_saves(self):
        factory = Factory()
        count = 5
        instances = factory.make(count, test_models.ModelWithNoFields)
        self.assertEqual(count,
                test_models.ModelWithNoFields.objects.all().count())

    def test_prepare_makes_many(self):
        factory = Factory()
        count = 5
        instances = factory.prepare(count, test_models.ModelWithNoFields)
        self.assertEqual(count, len(instances))

    def test_prepare_doesnt_save(self):
        factory = Factory()
        count = 5
        instances = factory.prepare(count, test_models.ModelWithNoFields)
        self.assertEqual(0,
                test_models.ModelWithNoFields.objects.all().count())

    def test_get_unique_integer_is_integer(self):
        factory = Factory()
        self.assertIsInstance(factory.getUniqueInteger(), int)

    def test_get_unique_integer_is_unique(self):
        factory = Factory()
        self.assertNotEquals(factory.getUniqueInteger(),
                factory.getUniqueInteger())

    def test_get_unique_string_is_str(self):
        factory = Factory()
        self.assertIsInstance(factory.getUniqueString(), str)

    def test_get_unique_string_is_unique(self):
        factory = Factory()
        self.assertNotEquals(factory.getUniqueString(),
                factory.getUniqueString())

    def test_get_unique_string_uses_prefix(self):
        factory = Factory()
        prefix = self.getUniqueString()
        self.assertThat(factory.getUniqueString(prefix=prefix),
                StartsWith(prefix))

    def test_get_unique_unicode_is_unicode(self):
        factory = Factory()
        self.assertIsInstance(factory.getUniqueUnicode(), unicode)

    def test_get_unique_unicode_is_unique(self):
        factory = Factory()
        self.assertNotEquals(factory.getUniqueUnicode(),
                factory.getUniqueUnicode())

    def test_get_unique_unicode_uses_prefix(self):
        factory = Factory()
        prefix = self.getUniqueString()
        self.assertThat(factory.getUniqueUnicode(prefix=prefix),
                StartsWith(prefix))

    def test_get_unique_date_is_date(self):
        factory = Factory()
        self.assertIsInstance(factory.getUniqueDate(), datetime.date)

    def test_get_unique_date_is_unique(self):
        factory = Factory()
        self.assertNotEquals(factory.getUniqueDate(), factory.getUniqueDate())

    def test_get_unique_datetime_is_datetime(self):
        factory = Factory()
        self.assertIsInstance(factory.getUniqueDateTime(), datetime.datetime)

    def test_get_unique_datetime_is_unique(self):
        factory = Factory()
        self.assertNotEquals(factory.getUniqueDateTime(), factory.getUniqueDateTime())

    def test_get_unique_time_is_time(self):
        factory = Factory()
        self.assertIsInstance(factory.getUniqueTime(), datetime.time)

    def test_get_unique_time_is_unique(self):
        factory = Factory()
        self.assertNotEquals(factory.getUniqueTime(), factory.getUniqueTime())

    def test__get_references_to_too_many_references(self):
        factory = Factory()
        source_model = test_models.ThrougModelWithTwoReferences
        target_model = test_models.ModelReferencingSelfWithThrough
        e = self.assertRaises(exceptions.ImproperlyConfigured,
                factory._get_references_to, source_model, target_model,
                'things', target_model)
        self.assertThat(str(e),
            StartsWith("Two references to %r from %r"
                % (target_model, source_model)))


class TestDummyTestCase(TestCase):

    def test_get_unique_integer_is_integer(self):
        dummy = DummyTestCase()
        self.assertIsInstance(dummy.getUniqueInteger(), int)

    def test_get_unique_integer_is_unique(self):
        dummy = DummyTestCase()
        self.assertNotEquals(dummy.getUniqueInteger(), dummy.getUniqueInteger())

    def test_get_unique_str_is_str(self):
        dummy = DummyTestCase()
        self.assertIsInstance(dummy.getUniqueString(), str)

    def test_get_unique_str_is_unique(self):
        dummy = DummyTestCase()
        self.assertNotEquals(dummy.getUniqueString(), dummy.getUniqueString())

    def test_get_unique_str_uses_prefix(self):
        dummy = DummyTestCase()
        prefix = self.getUniqueString()
        self.assertThat(dummy.getUniqueString(prefix=prefix), StartsWith(prefix))


class TestTestCases(TestCase):

    def run_test(self, cls, method_name):
        test = cls(method_name)
        result = ExtendedToOriginalDecorator(TestResult())
        result.startTestRun()
        test.run(result)
        result.stopTestRun()
        return result

    def test_test_case(self):
        class Test(testcase.TestCase):
            def test_foo(self):
                self.assertIsInstance(self.factory, Factory)
        result = self.run_test(Test, 'test_foo')
        self.assert_(result.wasSuccessful(), result.errors or result.failures)

    def test_transaction_test_case(self):
        class Test(testcase.TransactionTestCase):
            def test_foo(self):
                self.assertIsInstance(self.factory, Factory)
        result = self.run_test(Test, 'test_foo')
        self.assert_(result.wasSuccessful(), result.errors or result.failures)

    def test_test_case_get_unique_integer(self):
        class Test(testcase.TestCase):
            def test_foo(self):
                self.getUniqueInteger()
        result = self.run_test(Test, 'test_foo')
        self.assert_(result.wasSuccessful(), result.errors or result.failures)

    def test_transaction_test_case_get_unique_integer(self):
        class Test(testcase.TransactionTestCase):
            def test_foo(self):
                self.getUniqueInteger()
        result = self.run_test(Test, 'test_foo')
        self.assert_(result.wasSuccessful(), result.errors or result.failures)



# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf.urls.defaults import patterns, include, url

# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

urlpatterns = patterns('',
    # Examples:
    # url(r'^$', 'django_project.views.home', name='home'),
    # url(r'^django_project/', include('django_project.foo.urls')),

    # Uncomment the admin/doc line below to enable admin documentation:
    # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    # url(r'^admin/', include(admin.site.urls)),
)



# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.test import (
    TestCase as DjangoTestCase,
    TransactionTestCase as DjangoTransactionTestCase,
    )
from testtools import TestCase as TesttoolsTestCase

from django_factory.factory import Factory

__all__ = [
    'TestCase',
    'TransactionTestCase',
    ]


# N.B. The order of the superclasses matters here.
#  if Django is first then when running under 2.6
#  TesttoolsTestCase.__init__() won't be called,
#  causing all sorts of havoc.

class TestCase(TesttoolsTestCase, DjangoTestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        self.factory = Factory(test_case=self)


class TransactionTestCase(TesttoolsTestCase, DjangoTransactionTestCase):

    def setUp(self):
        super(TransactionTestCase, self).setUp()
        self.factory = Factory(test_case=self)



# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#!/usr/bin/env python
from django.core.management import execute_manager
import imp
try:
    imp.find_module('settings') # Assumed to be in the same directory.
except ImportError:
    import sys
    sys.stderr.write("Error: Can't find the file 'settings.py' in the directory containing %r. It appears you've customized things.\nYou'll have to run django-admin.py, passing it your settings module.\n" % __file__)
    sys.exit(1)

import settings

if __name__ == "__main__":
    execute_manager(settings)



# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Django settings for django_project project.

import django

django_ge_1_2 = False
django_ge_1_3 = False
if django.VERSION[0] > 1:
    django_ge_1_2 = True
    django_ge_1_3 = True
elif django.VERSION[1] > 1:
    django_ge_1_2 = True
    django_ge_1_3 = True
elif django.VERSION[1] > 2:
    django_ge_1_3 = True

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

if django_ge_1_3:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'django_factory.db',
        }
    }
else:
    DATABASE_ENGINE = 'sqlite3', # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
    DATABASE_NAME = 'django_factory.db',                      # Or path to database file if using sqlite3.

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = False

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = ''

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = ''

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# URL prefix for admin static files -- CSS, JavaScript and images.
# Make sure to use a trailing slash.
# Examples: "http://foo.com/static/admin/", "/static/admin/".
ADMIN_MEDIA_PREFIX = '/static/admin/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'q_m)oglvqisd7h4&psg2jf+_@s7(!ldbc-jvz81j_d3qpf6nrc'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'django_project.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

INSTALLED_APPS = (
    'django_factory',
    'django_project.factory_tests',
)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime

import django
from django.core.exceptions import ImproperlyConfigured
from django.db.models import fields as django_fields, ForeignKey, ManyToManyField
from django_factory import generators

__all__ = [
    'Factory',
    ]


class DummyTestCase(object):
    """A TestCase placeholder with the required methods.

    If the developer doesn't pass in a TestCase instance to the factory
    then we have to use something else to get unique values. This is
    that thing.
    """

    def __init__(self):
        self.counter = 0

    def getUniqueInteger(self):
        self.counter += 1
        return self.counter

    def getUniqueString(self, prefix=None):
        if prefix is None:
            prefix = ""
        return "%sstr-%d" % (prefix, self.getUniqueInteger())


def get_from_choices(choices):
    """Pick a value from the choices for a particular field."""
    return choices[0][0]


class Factory(object):

    # The name of the nested class that will be used to find any
    # generator functions for a model or its fields.
    MODEL_FACTORY_CLASS_NAME = 'Factory'

    # The default mapping from Fields to the functions that will
    # generate values for that field when needed.
    DEFAULT_FIELD_MAPPING = {
        django_fields.BooleanField: generators.generate_boolean,
        django_fields.NullBooleanField: generators.generate_boolean,

        django_fields.IntegerField: generators.generate_int,
        django_fields.SmallIntegerField: generators.generate_int,
        django_fields.PositiveIntegerField: generators.generate_int,
        django_fields.PositiveSmallIntegerField: generators.generate_int,
        django_fields.FloatField: generators.generate_float,
        django_fields.DecimalField: generators.generate_decimal,

        django_fields.CharField: generators.generate_unicode,
        django_fields.TextField: generators.generate_unicode,
        django_fields.SlugField: generators.generate_unicode,

        django_fields.DateField: generators.generate_date,
        django_fields.DateTimeField: generators.generate_datetime,
        django_fields.TimeField: generators.generate_time,

        django_fields.URLField: generators.generate_url,
        django_fields.EmailField: generators.generate_email,

        django_fields.CommaSeparatedIntegerField: generators.generate_comma_separated_integer,
        django_fields.IPAddressField: generators.generate_ip_address,

        django_fields.files.FileField: generators.generate_file,
        django_fields.files.ImageField: generators.generate_file,

        # TODO: FilePathField
        }

    # GenericIPAddressField is new in Django 1.4, so only reference it
    # if available.
    if getattr(django_fields, 'GenericIPAddressField', None) is not None:
        DEFAULT_FIELD_MAPPING[django_fields.GenericIPAddressField] = generators.generate_generic_ip_address

    # BigIntegerField is new in Django 1.2, so only reference it
    # if available.
    if getattr(django_fields, 'BigIntegerField', None) is not None:
        DEFAULT_FIELD_MAPPING[django_fields.BigIntegerField] = generators.generate_int

    DEFAULT_MODEL_GENERATORS = {}

    def __init__(self, test_case=None, field_mapping=None,
                 model_generators=None):
        """Create a Factory.

        :param test_case: a testtools.TestCase instance that should be used
            to get unique values when needed. May be None to generate unique
            values internally.
        :param field_mapping: the mapping from Field to function to generate
            values for that field to use in this factory. May be None
            to use the default mapping.
        :param model_generators: the mapping from Model to function to generate
            instances of that model to use in this factory. May be None
            to use the default mapping.
        """
        self.test_case = test_case or DummyTestCase()
        self.field_mapping = field_mapping or self.DEFAULT_FIELD_MAPPING.copy()
        self.model_generators = model_generators or self.DEFAULT_MODEL_GENERATORS.copy()

    def _get_model_factory_attribute(self, model, attr):
        """Get an attribute from ``model``'s nested Factory class, if any.

        :param model: the model to take the Factory class and attribute from.
        :param attr: the name of the attribute to receive.
        :return: the attribute or None.
        """
        model_factory = getattr(model, self.MODEL_FACTORY_CLASS_NAME, None)
        if model_factory:
            return getattr(model_factory, attr, None)
        return None

    def _get_instance_from_factory(self, model, attrs):
        """Get an instance from any factory instance generator on the model.

        :param model: the model to generate an instance for.
        :param attrs: the key/value dict containing developer-specified
            values for the instance attributes.
        :return: an instance of the model if the model's Factory nested
            class has an instance generator, or None if it does not.
        """
        instance = None
        instance_generator = self._get_model_factory_attribute(model,
                'make_instance')
        if instance_generator is not None:
            instance = instance_generator(self, **attrs)
        return instance

    def _number_of_many_to_many_to_make(self, model, field):
        """Returns the number of instances of a ManyToManyField target to make.

        Any factory nested class on the model will be consulted for an
        attribute specifying the number of instances to create. If the
        attribute is present then it will be returned or called as
        appropriate. If it is not present then the default value will be used.

        :param model: the model to check for a nested class property
                specifying the number of instances to create.
        :param field: the field that the value should correspond to.
        :return: an integer specifying the number of instances to create.
        """
        count = self._get_model_factory_attribute(model,
                "number_of_" + field.name)
        if count is not None:
            if callable(count):
                return count()
            else:
                return count
        return 1

    def _generate_many_to_many_value(self, model, field, not_including, attrs):
        """Generate a value for a ManyToManyField.

        :param model: the model that the field is being populated for.
        :param field: the particular field that the value is for.
        :param not_including: a list of Models that no value should be
                generated for in order to break loops.
        :param attrs: the key/value dict of developer-supplied values for the
                main instance.
        :return: a tuple of ``(generated, value)`` where ``generated`` is a
                boolean specifying whether any value was actually generated,
                and ``value`` is the generated value if ``generated`` is
                ``True``.
        """
        if field.name in attrs:
            return True, attrs.pop(field.name)
        else:
            if field.related.parent_model in not_including:
                return False, None
            has_factory, value = self._get_value_from_model_factory(model, field)
            if not has_factory:
                if field.null:
                    return False, None
                count = self._number_of_many_to_many_to_make(model, field)
                value = []
                for i in range(count):
                    value.append(self._create_one(field.related.parent_model, True,
                        not_including + [model], {}))
            return True, value

    def _generate_foreign_key_value(self, model, field, not_including, attrs):
        """Generate a value for a ForeignKey.

        :param model: the model that the field is being populated for.
        :param field: the particular field that the value is for.
        :param not_including: a list of Models that no value should be
                generated for in order to break loops.
        :param attrs: the key/value dict of developer-supplied values for the
                main instance.
        :return: a tuple of ``(generated, value)`` where ``generated`` is a
                boolean specifying whether any value was actually generated,
                and ``value`` is the generated value if ``generated`` is
                ``True``.
        """
        if field.name in attrs:
            return attrs[field.name] is not None, attrs.pop(field.name)
        else:
            if field.related.parent_model in not_including:
                return False, None
            has_factory, value = self._get_value_from_model_factory(model, field)
            if not has_factory:
                if field.blank and field.null:
                    return False, None
                # TODO: content types, maybe using a factory override
                #       for the model?
                value = self._create_one(field.related.parent_model, True,
                        not_including + [model], {})
            return True, value

    def _generate_plain_value(self, model, field, attrs):
        """Generate a value for a plain (non-reference) field..

        :param model: the model that the field is being populated for.
        :param field: the particular field that the value is for.
        :param attrs: the key/value dict of developer-supplied values for the
                main instance.
        :return: a tuple of ``(generated, value)`` where ``generated`` is a
                boolean specifying whether any value was actually generated,
                and ``value`` is the generated value if ``generated`` is
                ``True``.
        """
        if field.name in attrs:
            return False, None
        has_factory, value = self._get_value_from_model_factory(model, field)
        if not has_factory:
            if field.blank and field.null:
                return False, None
            if field.has_default():
                if callable(field.default):
                    value = field.default()
                else:
                    value = field.default
            else:
                value = self.generate_value(field)
        return True, value

    def _generate_instance(self, model, save, attrs, not_including):
        """Generate an instance of a model.

        :param model: the model that an instance should be generated for.
        :param attrs: the key/value dict of developer-supplied values for the
                instance attributes.
        :param not_including: a list of Models that no value should be
                generated for in order to break loops.
        :return: a tuple of ``(instance, many_to_may_dict, foreign_key_dict)``
                where ``instance`` is the generated instance,
                ``many_to_may_dict`` is a dict mapping from field name to
                list of instances for any ManyToManyFields and
                ``foreign_key_dict`` is a dict mapping from field name to
                instance for any ForeignKeys.
        """
        m2m_dict = {}
        fk_dict = {}
        fields = self.get_fields(model)
        for field in fields:
            if isinstance(field, ManyToManyField):
                if not save:
                    # When not saving the other instances aren't used so
                    # skip creating them.
                    continue
                use, value = self._generate_many_to_many_value(
                        model, field, not_including, attrs)
                if use:
                    m2m_dict[field.name] = value
            elif isinstance(field, ForeignKey):
                if not save:
                    # When not saving the other instances aren't used so
                    # skip creating them.
                    continue
                use, value = self._generate_foreign_key_value(
                        model, field, not_including, attrs)
                if use:
                    fk_dict[field.name] = value
            else:
                use, value = self._generate_plain_value(model, field, attrs)
                if use:
                    attrs[field.name] = value
        return model(**attrs), m2m_dict, fk_dict

    def _get_references_to(self, source_model, primary_model, field_name,
            secondary_model):
        """Get the foreign key fields referencing particular models.

        Given a source model and a primary and secondary target model,
        this method will return a tuple of the ForeignKey fields on the
        source model that reference the target models.

        :param source_model: the model to find the fields on.
        :param primary_model: the model that an instance is being
            generated for.
        :param field_name: the name of the ManyToManyField on the primary
            model that is being considered.
        :param secondary_model: the model that is the other side of the
            ManyToManyField.
        :return: a tuple of fields from source_model that refer to the target
                models, in the order (primary_model, secondary_model)
        :raises ImproperlyConfigured: if any target model is not referenced,
             or is referenced twice.
        """
        target_fields = [None, None]
        for field in self.get_fields(source_model):
            if isinstance(field, ForeignKey):
                for i, model in enumerate([primary_model, secondary_model]):
                    if field.related.parent_model == model:
                        if target_fields[i] is not None:
                            # FIXME: it would be nice to have a better
                            #        solution that having the developer
                            #        handle these cases by creating the
                            #        relations themselves.
                            raise ImproperlyConfigured(
                                ("Two references to %r from %r. The sematics "
                                 "of this relation are opaque to this code, "
                                 "so you must set number_of_%s = 0 on the "
                                 "Factory of %s and create the relationships "
                                 "youself.") % (model, source_model,
                                     field_name, primary_model))
                        target_fields[i] = field
        return target_fields

    def _generate_through_model_instances(self, through, instance,
            related_instances, not_including, field_name):
        """Generate instances of a through model for a ManyToManyField.

        When a ManyToManyField has a through model, relationships must be
        created by instantiating that through model. This method will
        create a number of instances of a through model given an instance
        of one side of the relationsip, and a list of instances of the
        other side. It will take care to set the appropriate attributes
        on the through model, regardless of their names, and generate
        values for any extra fields on the through model as needed.

        :param through: the intermediate model to create instances of.
        :param instance: the instance of one side of the relationship.
        :param related_instances: a list of instances of the other
            side of the relationship.
        :param not_including: a list of models to not create instances
            of when generating values for any extra fields on the
            through model.
        :param field_name: the name of the field on instance that
            the relations are being generated for.
        :return: None
        """
        if len(related_instances) < 1:
            return
        back_model = instance.__class__
        other_side_model = related_instances[0].__class__
        back_field, other_side_field = self._get_references_to(
                through, back_model, field_name, other_side_model)
        for related_instance in related_instances:
            self._create_one(through, True, not_including,
                    {back_field.name: instance,
                        other_side_field.name: related_instance})

    def _get_instance_from_model_generators(self, model, attrs):
        """Get an instance of ``model`` from the model_generators

        If ``model`` appears in model_generators use the generator
        from the mapping to create an instance of the model.
        The generator is passed the factory instance as the
        first parameter, and the attrs dict the develop
        supplied as **kwargs.

        :param model: the model to generate an instance of.
        :param attrs: the key/value dict the developer supplied to
            use as attributes on the generated model.
        :return: an instance of ``model`` or None if the
            model wasn't present in the model_generators.
        """
        if model in self.model_generators:
            return self.model_generators[model](self, **attrs)
        return None

    def _create_one(self, model, save, not_including, attrs):
        """Create one instance of a model, and optionally save it.

        :param model: the model that an instance should be generated for.
        :param save: whether to save the resulting instance.
        :param not_including: a list of Models that no value should be
                generated for in order to break loops.
        :param attrs: the key/value dict of developer-supplied values for the
                instance attributes.
        :return: the created instance.
        """
        m2m_dict = {}
        fk_dict = {}
        # XXX: do we have to pass ``save`` to the model_generators and
        # factory?
        instance = self._get_instance_from_model_generators(model, attrs)
        if instance is None:
            instance = self._get_instance_from_factory(model, attrs)
        if instance is None:
            instance, m2m_dict, fk_dict = self._generate_instance(model, save, attrs, not_including)
        if save:
            for key, model_instance in fk_dict.items():
                model_instance.save()
                setattr(instance, key, model_instance)

            instance.save()

            for key, value in m2m_dict.items():
                if len(value) < 1:
                    break
                instance_relation = getattr(instance, key)
                through_model = None
                if django.VERSION < (1, 2):
                    if instance_relation.through is not None:
                        through_model = instance_relation.instance._meta.get_field(key).rel.through_model
                else:
                    if not instance_relation.through._meta.auto_created:
                        through_model = instance_relation.through
                if through_model is None:
                    for model_instance in value:
                        model_instance.save()
                        instance_relation.add(model_instance)
                else:
                    self._generate_through_model_instances(
                            through_model, instance, value,
                            not_including, key)
            if getattr(instance, 'full_clean', None) is not None:
                instance.full_clean()
        return instance

    def _many(self, count, method, model, attrs):
        """Create a number of instances using the provided method."""
        instances = []
        for i in range(count):
            instances.append(method(model, **attrs))
        return instances

    def make(self, count, model, **attrs):
        """Make a configurable number of instances of ``model``.

        :param count: the number of instances to make
        :type count: int
        :param model: the model to create instances of
        :type model: ``Model``
        :param **kwargs: any attribute values to set to known values.
        :return: a list of created instances, the length of which
                will equal ``count``. All instances will be saved and
                have attributes defined by ``attrs``.
        """
        return self._many(count, self.make_one, model, attrs)

    def prepare(self, count, model, **attrs):
        """Prepare a configurable number of instances of ``model``.

        Note that ManyToManyFields and ForeignKeys will be
        None, as they can't be added unless the instance is saved.
        You will have to create instances of the referenced models
        and assign them to attributes as needed.

        :param count: the number of instances to prepare
        :type count: int
        :param model: the model to prepare instances of
        :type model: ``Model``
        :param **kwargs: any attribute values to set to known values.
        :return: a list of created instances, the length of which
                will equal ``count``. All instances will not be saved
                and have attributes defined by ``attrs``.
        """
        return self._many(count, self.prepare_one, model, attrs)

    def make_one(self, model, **attrs):
        """Make an instance of ``model``.

        :param model: the model to make instances of
        :type model: ``Model``
        :param **kwargs: any attribute values to set to known values.
        :return: an instance of ``model``.  The instance will be saved
                and have attributes defined by ``attrs``.
        """
        return self._create_one(model, True, [], attrs)

    def prepare_one(self, model, **attrs):
        """Prepare an instance of ``model``.

        Note that ManyToManyFields and ForeignKeys will be
        None, as they can't be added unless the instance is saved.
        You will have to create instances of the referenced models
        and assign them to attributes as needed.

        :param model: the model to prepare instances of
        :type model: ``Model``
        :param **kwargs: any attribute values to set to known values.
        :return: an instance of ``model``.  The instance will not be saved
                and have attributes defined by ``attrs``.
        """
        return self._create_one(model, False, [], attrs)

    def _get_value_from_model_factory(self, model, field):
        """Get the value for a field from a factory generator, if any.

        If the model has a factory generator for the field, then
        call it to get the value that should be used.

        :param model: the model to check the factory nested class of.
        :param field: the field that the value should be generated for.
        :return: a tuple of (``generated``, ``value``) where ``generated``
                is a boolean specifying whether a function was present,
                and ``value`` will be the generated value if ``generated``
                is ``True``.
        """
        generator = self._get_model_factory_attribute(model,
                "get_" + field.name)
        if generator:
            return True, generator(field, self)
        return False, None

    def generate_value(self, field):
        """Generate a value for ``field`` on ``model``.

        This method will generate a value for ``field`` depending
        on its type, and any attributes it has (such as ``choices``.)
        It does not obey any model factory nested class, or
        attributes such as ``default`` or ``null``.

        :param field: the field instance to generate the value for.
        :return: the value that should be used.
        :raises TypeError: if no generator is registered for the
            type of ``field``.
        """
        if getattr(field, 'choices'):
            generator = lambda x, y: get_from_choices(field.flatchoices)
        else:
            generator = self.field_mapping.get(field.__class__)
        if generator is None:
            raise TypeError("Can't generate value for field: %s" % field)
        return generator(field, self)

    @staticmethod
    def get_fields(model):
        """Get the fields of ``model`` filtering any automatic fields.

        :param model: the Model to get the fields of.
        :return: a list of field instances, not including any ``AutoField``.
        """
        fields = model._meta.fields[:] + model._meta.many_to_many[:]
        for field in fields[:]:
            # TODO: filter GenericRelation
            if isinstance(field, django_fields.AutoField):
                fields.remove(field)
        return fields

    def getUniqueInteger(self):
        """Get a unique integer.

        The value will be unique for this factory.

        :return_type: int
        """
        return self.test_case.getUniqueInteger()

    def getUniqueString(self, prefix=None):
        """Get a unique string.

        The value will be unique for this factory.

        :return_type: str
        """
        return self.test_case.getUniqueString(prefix=prefix)

    def getUniqueUnicode(self, prefix=None):
        """Get a unique unicode string.

        The value will be unique for this factory.

        :return_type: unicode
        """
        return self.getUniqueString(prefix=prefix).decode("utf-8")

    def getUniqueDate(self):
        """Get a unique date.

        The value will be unique for this factory.

        :return_type: datetime.date
        """
        start_date = datetime.date(2000, 1, 1)
        return start_date + datetime.timedelta(days=self.getUniqueInteger())

    def getUniqueDateTime(self):
        """Get a unique datetime.

        The value will be unique for this factory.

        :return_type: datetime.datetime
        """
        start_date = datetime.datetime(1970, 1, 1)
        return start_date + datetime.timedelta(days=self.getUniqueInteger())

    def getUniqueTime(self):
        """Get a unique time.

        The value will be unique for this factory.

        :return_type: datetime.time
        """
        start_date = self.getUniqueDateTime()
        return (start_date + datetime.timedelta(minutes=self.getUniqueInteger())).time()



# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from decimal import Decimal
import sys

from django.core.files.base import ContentFile


def generate_boolean(field, factory):
    """Generate a boolean.

    Always returns False.

    :returns: False
    """
    return False


def generate_unicode(field, factory):
    """Generate a unicode sting.

    :returns: a unique string starting with the field name.
    :return_type: unicode
    """
    value = factory.getUniqueUnicode(prefix=field.name)
    max_length = getattr(field, "max_length", sys.maxint)
    if max_length is not None and len(value) > max_length:
        value = value[-max_length:]
    return value


def generate_int(field, factory):
    """Generate an int.

    :returns: a unique integer.
    :return_type: int
    """
    return factory.getUniqueInteger()


def generate_float(field, factory):
    """Generate a float.

    :returns: a unique float.
    :return_type: float
    """
    return factory.getUniqueInteger() + 0.1


def generate_decimal(field, factory):
    """Generate a Decimal.

    :returns: a unique Decimal.
    :return_type: Decimal
    """
    return Decimal(factory.getUniqueInteger())


def generate_date(field, factory):
    """Generate a date.

    :returns: a unique datetime.date.
    :return_type: datetime.date
    """
    return factory.getUniqueDate()


def generate_datetime(field, factory):
    """Generate a datetime.

    :returns: a unique datetime.datetime.
    :return_type: datetime.datetime
    """
    return factory.getUniqueDateTime()


def generate_time(field, factory):
    """Generate a time.

    :returns: a unique datetime.time.
    :return_type: datetime.time
    """
    return factory.getUniqueTime()


def generate_url(field, factory):
    """Generate a url.

    :returns: a unique url.
    :return_type: unicode
    """
    return u"http://example.com/" + generate_unicode(field, factory)


def generate_email(field, factory):
    """Generate an email.

    :returns: a unique email.
    :return_type: unicode
    """
    return generate_unicode(field, factory) + '@example.com'


def generate_comma_separated_integer(field, factory):
    """Generate an comma separated integer string.

    :returns: a unique comma separated integer string.
    :return_type: str
    """
    return "%d" % generate_int(field, factory)


def generate_ip_address(field, factory):
    """Generate an IP address.

    :returns: a unique IP address.
    :return_type: str
    """
    parts = []
    for i in range(4):
        parts.append(str(generate_int(field, factory) % 256))
    return ".".join(parts)


def generate_generic_ip_address(field, factory):
    """Generate a generic IP address.

    :returns: a unique IPv6 address.
    :return_type: str
    """
    parts = []
    for i in range(2):
        parts.append("%x" % (generate_int(field, factory) % 256, ))
    return "::".join(parts)


def generate_file(field, factory):
    # FIXME: use a File rather than a ContentFile so that the other
    # attributes are available
    return ContentFile(generate_unicode(field, factory).encode("utf-8"))



# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.core import exceptions
from django.db import models


class ModelWithNoFields(models.Model):
    pass


class ModelWithCharField(models.Model):

    char_field = models.CharField(max_length=100)


class ModelWithNullAndBlankCharField(models.Model):

    char_field = models.CharField(max_length=100, null=True, blank=True)


class ModelWithNullCharField(models.Model):

    char_field = models.CharField(max_length=100, null=True)


class ModelWithBlankCharField(models.Model):

    char_field = models.CharField(max_length=100, blank=True)


class ModelWithMaxLengthCharField(models.Model):

    char_field = models.CharField(max_length=5)


class ModelWithNullAndBlankCharFieldAndFactory(models.Model):

    VALUE = 'bar'

    char_field = models.CharField(max_length=100, null=True, blank=True)

    class Factory:

        @staticmethod
        def get_char_field(field, factory):
            return ModelWithNullAndBlankCharFieldAndFactory.VALUE


class ModelWithDefaultCharField(models.Model):

    DEFAULT = 'foo'

    char_field = models.CharField(max_length=100, default=DEFAULT)


class ModelWithDefaultCharFieldAndFactory(models.Model):

    VALUE = 'bar'
    DEFAULT = 'foo'

    char_field = models.CharField(max_length=100, default=DEFAULT)

    class Factory:

        @staticmethod
        def get_char_field(field, factory):
            return ModelWithDefaultCharFieldAndFactory.VALUE


class ModelWithCallableDefaultCharField(models.Model):

    DEFAULT = 'foo'

    def get_default():
        return ModelWithCallableDefaultCharField.DEFAULT

    char_field = models.CharField(max_length=100, default=get_default)


class ModelWithTextField(models.Model):

    text_field = models.TextField(max_length=100)


class ModelWithNoneMaxLengthTextField(models.Model):

    text_field = models.TextField(max_length=None)


class ModelWithSlugField(models.Model):

    slug_field = models.SlugField(max_length=100)


class ModelWithBooleanField(models.Model):

    bool_field = models.BooleanField()


class ModelWithNullBooleanField(models.Model):

    bool_field = models.NullBooleanField()


class ModelWithIntegerField(models.Model):

    int_field = models.IntegerField()


if getattr(models, 'BigIntegerField', None) is not None:
    class ModelWithBigIntegerField(models.Model):

        big_int_field = models.BigIntegerField()


class ModelWithSmallIntegerField(models.Model):

    small_int_field = models.SmallIntegerField()


class ModelWithPositiveIntegerField(models.Model):

    positive_int_field = models.PositiveIntegerField()


class ModelWithPositiveSmallIntegerField(models.Model):

    positive_small_int_field = models.PositiveSmallIntegerField()


class ModelWithFloatField(models.Model):

    float_field = models.FloatField()


class ModelWithDecimalField(models.Model):

    decimal_field = models.DecimalField(decimal_places=2, max_digits=3)


class ModelWithDateField(models.Model):

    date_field = models.DateField()


class ModelWithDateTimeField(models.Model):

    datetime_field = models.DateTimeField()


class ModelWithTimeField(models.Model):

    time_field = models.TimeField()


class ModelWithURLField(models.Model):

    url_field = models.URLField()


class ModelWithEmailField(models.Model):

    email_field = models.EmailField()


class ModelWithCommaSeparatedIntegerField(models.Model):

    csi_field = models.CommaSeparatedIntegerField(max_length=6)


class ModelWithIPAddressField(models.Model):

    ip_field = models.IPAddressField()


if getattr(models, 'GenericIPAddressField', None) is not None:
    class ModelWithGenericIPAddressField(models.Model):

        ip_field = models.GenericIPAddressField()


class ModelWithFileField(models.Model):

    file_field = models.FileField(upload_to='foo')


class ModelWithImageField(models.Model):

    image_field = models.ImageField(upload_to='foo')


class ModelWithChoices(models.Model):

    CHOICES = [(-1, 'foo'), (-2, 'bar')]

    choices_field = models.IntegerField(choices=CHOICES)


class ModelWithChoiceGroups(models.Model):

    CHOICES = [('foo', ((-1, 'foo'), )), ('bar', ((-2, 'bar'), ))]

    choices_field = models.IntegerField(choices=CHOICES)


class ModelWithFieldGenerator(models.Model):

    VALUE = 'bar'

    char_field = models.CharField(max_length=100)

    class Factory:

        @staticmethod
        def get_char_field(field, factory):
            return ModelWithFieldGenerator.VALUE


class ModelWithInstanceGenerator(models.Model):

    VALUE = 'bar'

    char_field = models.CharField(max_length=100)

    class Factory:

        @staticmethod
        def make_instance(factory, char_field=None):
            if char_field is None:
                char_field = ModelWithInstanceGenerator.VALUE
            return ModelWithInstanceGenerator(char_field=char_field)


class ModelReferencedByManyToManyField(models.Model):
    pass


class ModelWithManyToManyField(models.Model):

    things = models.ManyToManyField(ModelReferencedByManyToManyField)


class ModelWithManyToManyFieldAndFactory(models.Model):

    VALUE = ModelReferencedByManyToManyField()

    things = models.ManyToManyField(ModelReferencedByManyToManyField)

    class Factory:

        @staticmethod
        def get_things(field, factory):
            return [ModelWithManyToManyFieldAndFactory.VALUE]


class ModelWithNullManyToManyField(models.Model):

    things = models.ManyToManyField(ModelReferencedByManyToManyField, null=True)


class ModelWithNullManyToManyFieldAndFactory(models.Model):

    VALUE = ModelReferencedByManyToManyField()

    things = models.ManyToManyField(ModelReferencedByManyToManyField, null=True)

    class Factory:

        @staticmethod
        def get_things(field, factory):
            return [ModelWithNullManyToManyFieldAndFactory.VALUE]


class ModelWithSelfManyToManyField(models.Model):

    # Not symetrical so we can tell which is the child
    things = models.ManyToManyField('self', symmetrical=False)


class ModelWithCircularReferenceLoop1(models.Model):

    things = models.ManyToManyField('ModelWithCircularReferenceLoop2')


class ModelWithCircularReferenceLoop2(models.Model):

    other_things = models.ManyToManyField('ModelWithCircularReferenceLoop1')


class ModelWantingTwoManyToManyInstances(models.Model):

    things = models.ManyToManyField(ModelReferencedByManyToManyField)

    class Factory:
        number_of_things = 2


class ModelWantingZeroManyToManyInstances(models.Model):

    things = models.ManyToManyField(ModelReferencedByManyToManyField)

    class Factory:
        number_of_things = 0


class ModelWithCallableSpecifiedManyToManyField(models.Model):

    COUNT = 2

    things = models.ManyToManyField(ModelReferencedByManyToManyField)

    class Factory:

        @staticmethod
        def number_of_things():
            return ModelWithCallableSpecifiedManyToManyField.COUNT


###
# We break the naming convention for these models so the created
# attributes have reasonable names
class Person(models.Model):
    pass


class Membership(models.Model):

    person = models.ForeignKey(Person)
    group = models.ForeignKey('Group')
    comment = models.CharField(max_length=100)


class Group(models.Model):

    people = models.ManyToManyField(Person, through=Membership)
###


class ThrougModelWithTwoReferences(models.Model):

    one = models.ForeignKey('ModelReferencingSelfWithThrough',
            related_name="ones")
    two = models.ForeignKey('ModelReferencingSelfWithThrough',
            related_name="twos")


class ModelReferencingSelfWithThrough(models.Model):

    things = models.ManyToManyField('self',
            through=ThrougModelWithTwoReferences, symmetrical=False)


class ModelReferencedByForiegnKeyField(models.Model):
    pass


class ModelWithForeignKey(models.Model):

    thing = models.ForeignKey(ModelReferencedByForiegnKeyField)


class ModelWithNullAndBlankForeignKey(models.Model):

    thing = models.ForeignKey(ModelReferencedByForiegnKeyField, null=True, blank=True)


class ModelWithNullForeignKey(models.Model):

    thing = models.ForeignKey(ModelReferencedByForiegnKeyField, null=True)


class ModelWithBlankForeignKey(models.Model):

    thing = models.ForeignKey(ModelReferencedByForiegnKeyField, blank=True)


class ModelWithForeignKeyAndFactory(models.Model):

    VALUE = ModelReferencedByForiegnKeyField()

    thing = models.ForeignKey(ModelReferencedByForiegnKeyField)

    class Factory:

        @staticmethod
        def get_thing(field, factory):
            return ModelWithForeignKeyAndFactory.VALUE


class ModelWithSelfForeignKey(models.Model):

    thing = models.ForeignKey('self', null=True, blank=True)


class ModelWithCircularForiegnKeyReferenceLoop1(models.Model):

    thing = models.ForeignKey('ModelWithCircularForiegnKeyReferenceLoop2')


class ModelWithCircularForiegnKeyReferenceLoop2(models.Model):

    thing = models.ForeignKey('ModelWithCircularForiegnKeyReferenceLoop1', null=True, blank=True)


class ModelWithForiegnKeyToModelWithForeignKey(models.Model):

    thing = models.ForeignKey(ModelWithForeignKey)


class ModelWithForeignKeyAndSelfManyToMany(models.Model):

    thing = models.ForeignKey(ModelReferencedByForiegnKeyField)
    selves = models.ManyToManyField('self', symmetrical=False)


if getattr(models.CharField, 'default_validators', None) is not None:
    class ModelWithImpossibleValidator(models.Model):

        def impossible(value):
            raise exceptions.ValidationError('Impossible')

        char_field = models.CharField(max_length=100, validators=[impossible])



#!/usr/bin/env python
# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
        name='django_factory',
        version='0.10',
        description=('Generic factory for creating instances of Django '
                     'models in tests.'),
        author='James Westby',
        author_email='james.westby@canonical.com',
        url='https://launchpad.net/django_factory',
        packages=['django_factory'],
        install_requires=['django>=1.1', 'testtools'],
        license="Apache-2",
        long_description=read('README'),
        classifiers=[
            'Development Status :: 4 - Beta',
            'Framework :: Django',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python :: 2.6',
            'Topic :: Database',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Software Development :: Testing',
        ],
)
